import logging
import time
from typing import Dict, Any, Optional
import signal
import sys
import os

from .utils.logger import Logger
from .utils.config import Config
from .utils.monitoring import Monitoring
from .protocols.protocol_factory import ProtocolFactory
from .services.file_sender import FileSender
from .services.file_fetcher import FileFetcher


class Application:
    """
    Main application class for SSH/FTP file transfer
    """

    def __init__(self, config_path: Optional[str] = None):
        # Initialize logging
        self.logger_util = Logger("ssh_file_to_cbs")
        self.logger = self.logger_util.get_logger()

        # Load configuration
        self.config = Config(config_path)

        # Initialize monitoring
        self.monitoring = Monitoring("SSHFileToCbs")

        # Set up signal handlers
        signal.signal(signal.SIGINT, self._signal_handler)
        signal.signal(signal.SIGTERM, self._signal_handler)

        self.running = False
        self.logger.info("Application initialized")

    def _signal_handler(self, sig, frame):
        """Handle termination signals"""
        self.logger.info(f"Received signal {sig}, shutting down...")
        self.running = False

    def _create_protocol(self) -> Any:
        """Create protocol instance from configuration"""
        try:
            protocol_type = self.config.get("app", "TRANSFER_PROTOCOL")
            host = self.config.get("server", "REMOTE_HOST")
            username = self.config.get("server", "REMOTE_USER")
            password = self.config.get("server", "REMOTE_PASS")
            port = self.config.get_int("server", "REMOTE_PORT")

            if not all([protocol_type, host, username, password, port]):
                self.logger.error("Missing required configuration for protocol")
                return None

            protocol = ProtocolFactory.create_protocol(
                protocol_type, host, username, password, port
            )

            if not protocol:
                self.logger.error(f"Failed to create protocol: {protocol_type}")
                return None

            return protocol

        except Exception as e:
            self.logger.error(f"Error creating protocol: {str(e)}")
            return None

    def send_files(self) -> bool:
        """Send files from local to remote server"""
        try:
            protocol = self._create_protocol()
            if not protocol:
                return False

            local_folder_path = self.config.get("paths", "LOCAL_FOLDER_PATH")
            archive_folder_path = self.config.get("paths", "ARCHIVE_FOLDER_PATH")
            remote_input_path = self.config.get(
                "remote_paths", "REMOTE_INPUT_FILE_PATH"
            )

            if not all([local_folder_path, archive_folder_path, remote_input_path]):
                self.logger.error("Missing required configuration for sending files")
                return False

            sender = FileSender(
                protocol, local_folder_path, remote_input_path, archive_folder_path
            )

            files_sent = sender.send_files()

            self.monitoring.record_operation(
                "send_files",
                "success" if files_sent > 0 else "no_files",
                {"files_sent": files_sent},
            )

            return True

        except Exception as e:
            self.logger.error(f"Error in send_files: {str(e)}")
            self.monitoring.record_operation("send_files", "failure", {"error": str(e)})
            return False

    def fetch_report_files(self) -> bool:
        """Fetch report files from remote server"""
        try:
            protocol = self._create_protocol()
            if not protocol:
                return False

            local_report_path = self.config.get("paths", "LOCAL_REPORT_PATH")
            remote_output_path = self.config.get(
                "remote_paths", "REMOTE_OUTPUT_FILE_PATH"
            )
            report_pattern = self.config.get("remote_paths", "REMOTE_REPORT_PATTERN")

            if not all([local_report_path, remote_output_path]):
                self.logger.error(
                    "Missing required configuration for fetching report files"
                )
                return False

            fetcher = FileFetcher(
                protocol, remote_output_path, local_report_path, report_pattern
            )

            files_fetched = fetcher.fetch_files()

            self.monitoring.record_operation(
                "fetch_report_files",
                "success" if files_fetched else "no_files",
                {"files_fetched": len(files_fetched)},
            )

            return True

        except Exception as e:
            self.logger.error(f"Error in fetch_report_files: {str(e)}")
            self.monitoring.record_operation(
                "fetch_report_files", "failure", {"error": str(e)}
            )
            return False

    def fetch_failed_files(self) -> bool:
        """Fetch failed files from remote server"""
        try:
            protocol = self._create_protocol()
            if not protocol:
                return False

            local_failed_path = self.config.get("paths", "LOCAL_FAILED_PATH")
            remote_failure_path = self.config.get(
                "remote_paths", "REMOTE_FAILURE_FILE_PATH"
            )
            report_pattern = self.config.get("remote_paths", "REMOTE_REPORT_PATTERN")

            if not all([local_failed_path, remote_failure_path]):
                self.logger.error(
                    "Missing required configuration for fetching failed files"
                )
                return False

            fetcher = FileFetcher(
                protocol, remote_failure_path, local_failed_path, report_pattern
            )

            files_fetched = fetcher.fetch_files()

            self.monitoring.record_operation(
                "fetch_failed_files",
                "success" if files_fetched else "no_files",
                {"files_fetched": len(files_fetched)},
            )

            return True

        except Exception as e:
            self.logger.error(f"Error in fetch_failed_files: {str(e)}")
            self.monitoring.record_operation(
                "fetch_failed_files", "failure", {"error": str(e)}
            )
            return False

    def run(self) -> None:
        """Run the application main loop"""
        self.logger.info("Starting application main loop")
        self.running = True

        # Start background monitoring
        self.monitoring.start_background_monitoring()

        sleep_time = self.config.get_int("app", "SLEEP_TIME_MINS", fallback=30)
        self.logger.info(f"Sleep time between cycles: {sleep_time} minutes")

        try:
            while self.running:
                self.logger.info("Starting file transfer cycle")

                # Send files to remote server
                self.send_files()

                # Fetch report files
                self.fetch_report_files()

                # Fetch failed files
                self.fetch_failed_files()

                self.logger.info(
                    f"File transfer cycle completed, sleeping for {sleep_time} minutes"
                )

                # Sleep but check periodically if we should continue running
                for _ in range(sleep_time * 60 // 10):
                    if not self.running:
                        break
                    time.sleep(10)

        except Exception as e:
            self.logger.error(f"Error in main loop: {str(e)}")
        finally:
            self.monitoring.stop_background_monitoring()
            self.logger.info("Application shutdown complete")

