import os
import shutil
import logging
from typing import List, Optional
import glob

class FileManager:
    """
    Handles local file operations like moving files to archive
    """
    def __init__(self):
        self.logger = logging.getLogger("SSHFileToCbs.FileManager")
    
    def move_file(self, source_path: str, target_dir: str) -> bool:
        """
        Move a file from source path to target directory
        
        Args:
            source_path: Path to source file
            target_dir: Directory to move file to
        
        Returns:
            True if successful, False otherwise
        """
        try:
            if not os.path.isfile(source_path):
                self.logger.error(f"Source file not found: {source_path}")
                return False
                
            # Create target directory if it doesn't exist
            os.makedirs(target_dir, exist_ok=True)
            
            # Get filename
            filename = os.path.basename(source_path)
            target_path = os.path.join(target_dir, filename)
            
            # Move the file
            shutil.move(source_path, target_path)
            self.logger.info(f"Moved file from {source_path} to {target_path}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to move file {source_path} to {target_dir}: {str(e)}")
            return False
    
    def get_files_in_directory(self, directory: str, pattern: Optional[str] = None) -> List[str]:
        """
        Get list of files in directory, optionally filtered by pattern
        
        Args:
            directory: Directory to list files from
            pattern: Optional glob pattern to filter files
        
        Returns:
            List of file paths
        """
        try:
            if not os.path.isdir(directory):
                self.logger.error(f"Directory not found: {directory}")
                return []
            
            # Get all files in directory
            if pattern:
                file_pattern = os.path.join(directory, pattern)
                files = glob.glob(file_pattern)
            else:
                files = [os.path.join(directory, f) for f in os.listdir(directory) 
                         if os.path.isfile(os.path.join(directory, f))]
            
            self.logger.debug(f"Found {len(files)} files in {directory}")
            return files
            
        except Exception as e:
            self.logger.error(f"Error listing files in {directory}: {str(e)}")
            return []