import os
import ftplib
import fnmatch
from typing import List, Optional
import logging

from .protocol_interface import FileTransferProtocol

class FTPProtocol(FileTransferProtocol):
    """
    Implementation of FTP file transfer protocol
    """
    def __init__(self, host: str, username: str, password: str, port: int = 21):
        super().__init__(host, username, password, port)
        self.client = None
        self.connected = False
        
    def connect(self) -> bool:
        """
        Establish FTP connection
        """
        try:
            self.logger.info(f"Connecting to FTP server {self.username}@{self.host}:{self.port}")
            self.client = ftplib.FTP()
            self.client.connect(self.host, self.port, timeout=30)
            self.client.login(self.username, self.password)
            self.client.set_pasv(True)  # Use passive mode
            self.connected = True
            self.logger.info("FTP connection established successfully")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to establish FTP connection: {str(e)}")
            self.disconnect()
            return False
    
    def disconnect(self) -> None:
        """
        Close FTP connection
        """
        try:
            if self.client:
                self.client.quit()
                self.client = None
                
            self.connected = False
            self.logger.info("FTP connection closed")
            
        except Exception as e:
            self.logger.error(f"Error disconnecting FTP: {str(e)}")
    
    def _ensure_connection(self) -> bool:
        """
        Ensure connection is established
        """
        if not self.connected or not self.client:
            return self.connect()
        return True
    
    def _ensure_remote_dir(self, remote_dir_path: str) -> bool:
        """
        Ensure remote directory exists, create if it doesn't
        """
        if not self._ensure_connection():
            return False
            
        try:
            # Get current directory to return to later
            original_dir = self.client.pwd()
            
            # Split path and create each directory component
            dirs = remote_dir_path.strip('/').split('/')
            
            for i, directory in enumerate(dirs):
                if not directory:
                    continue
                
                try:
                    # Try to change to this directory
                    self.client.cwd(directory)
                except ftplib.error_perm:
                    # If it doesn't exist, create it
                    try:
                        self.logger.info(f"Creating remote directory: {directory}")
                        self.client.mkd(directory)
                        self.client.cwd(directory)
                    except Exception as e:
                        self.logger.error(f"Failed to create directory {directory}: {str(e)}")
                        # Return to original directory
                        self.client.cwd(original_dir)
                        return False
            
            # Return to original directory
            self.client.cwd(original_dir)
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to ensure remote directory {remote_dir_path}: {str(e)}")
            return False
    
    def send_file(self, local_file_path: str, remote_dir_path: str) -> bool:
        """
        Send file to remote server
        
        Args:
            local_file_path: Path to local file
            remote_dir_path: Directory on remote server
        
        Returns:
            True if successful, False otherwise
        """
        if not self._ensure_connection():
            return False
            
        try:
            # Ensure local file exists
            if not os.path.isfile(local_file_path):
                self.logger.error(f"Local file not found: {local_file_path}")
                return False
            
            # Get filename and create branchwise directory
            filename = os.path.basename(local_file_path)
            file_tokens = filename.split('-', 1)
            
            if len(file_tokens) < 2:
                self.logger.warning(f"Filename {filename} doesn't match expected format (branch-filename)")
                branch_dir = "default"
                remote_filename = filename
            else:
                branch_dir = file_tokens[0]
                remote_filename = file_tokens[1]
            
            # Ensure remote directory structure exists
            branch_path = f"{remote_dir_path.rstrip('/')}/{branch_dir}"
            if not self._ensure_remote_dir(branch_path):
                return False
            
            # Change to target directory
            self.client.cwd(branch_path)
            
            # Upload the file
            self.logger.info(f"Uploading {local_file_path} to {branch_path}/{remote_filename}")
            with open(local_file_path, 'rb') as file:
                self.client.storbinary(f'STOR {remote_filename}', file)
            
            self.logger.info(f"File {filename} uploaded successfully")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to upload file {local_file_path}: {str(e)}")
            return False
    
    def fetch_files(self, remote_dir_path: str, local_dir_path: str, pattern: Optional[str] = None) -> List[str]:
        """
        Fetch files from remote server
        
        Args:
            remote_dir_path: Directory on remote server
            local_dir_path: Directory to store downloaded files
            pattern: Optional file pattern to match
        
        Returns:
            List of successfully downloaded files
        """
        if not self._ensure_connection():
            return []
            
        downloaded_files = []
        
        try:
            # Ensure local directory exists
            os.makedirs(local_dir_path, exist_ok=True)
            
            # Try to change to remote directory
            try:
                self.client.cwd(remote_dir_path)
            except ftplib.error_perm as e:
                self.logger.warning(f"Remote directory not found or permission denied: {remote_dir_path}")
                return []
            
            # Get list of files
            file_list = []
            self.client.retrlines('LIST', lambda x: file_list.append(x))
            
            # Process files
            for file_info in file_list:
                # Skip directories
                if file_info.startswith('d'):
                    continue
                
                # Extract filename (last part of the listing)
                parts = file_info.split()
                if len(parts) < 9:
                    continue
                
                filename = ' '.join(parts[8:])
                
                # Skip if pattern is specified and doesn't match
                if pattern and not fnmatch.fnmatch(filename, pattern):
                    continue
                
                local_file_path = os.path.join(local_dir_path, filename)
                
                # Only download if file doesn't exist locally
                if not os.path.exists(local_file_path):
                    self.logger.info(f"Downloading {filename} to {local_file_path}")
                    
                    try:
                        with open(local_file_path, 'wb') as file:
                            self.client.retrbinary(f'RETR {filename}', file.write)
                        downloaded_files.append(local_file_path)
                    except Exception as e:
                        self.logger.error(f"Failed to download {filename}: {str(e)}")
                else:
                    self.logger.debug(f"File already exists locally: {local_file_path}")
            
            return downloaded_files
            
        except Exception as e:
            self.logger.error(f"Failed to fetch files from {remote_dir_path}: {str(e)}")
            return downloaded_files