import logging
from typing import Optional

from .protocol_interface import FileTransferProtocol
from .ssh_protocol import SSHProtocol
from .ftp_protocol import FTPProtocol

class ProtocolFactory:
    """
    Factory class to create appropriate protocol instances
    """
    @staticmethod
    def create_protocol(protocol_type: str, host: str, username: str, password: str, port: int) -> Optional[FileTransferProtocol]:
        """
        Create and return a protocol instance based on protocol type
        
        Args:
            protocol_type: Type of protocol ('SSH' or 'FTP')
            host: Remote host address
            username: Username for authentication
            password: Password for authentication
            port: Port number
        
        Returns:
            FileTransferProtocol instance or None if invalid protocol type
        """
        logger = logging.getLogger("SSHFileToCbs.ProtocolFactory")
        
        protocol_type = protocol_type.upper()
        
        if protocol_type == "SSH":
            logger.info(f"Creating SSH protocol for {username}@{host}:{port}")
            return SSHProtocol(host, username, password, port)
            
        elif protocol_type == "FTP":
            logger.info(f"Creating FTP protocol for {username}@{host}:{port}")
            return FTPProtocol(host, username, password, port)
            
        else:
            logger.error(f"Invalid protocol type: {protocol_type}")
            return None