from abc import ABC, abstractmethod
from typing import List, Optional
import logging

class FileTransferProtocol(ABC):
    """
    Abstract base class that defines the interface for file transfer protocols
    """
    def __init__(self, host: str, username: str, password: str, port: int):
        self.host = host
        self.username = username
        self.password = password
        self.port = port
        self.logger = logging.getLogger(f"SSHFileToCbs.{self.__class__.__name__}")
    
    @abstractmethod
    def connect(self) -> bool:
        """
        Establish connection to remote server
        Returns True if connection successful, False otherwise
        """
        pass
    
    @abstractmethod
    def disconnect(self) -> None:
        """
        Close connection to remote server
        """
        pass
    
    @abstractmethod
    def send_file(self, local_file_path: str, remote_dir_path: str) -> bool:
        """
        Send file to remote server
        
        Args:
            local_file_path: Path to local file
            remote_dir_path: Directory on remote server
        
        Returns:
            True if successful, False otherwise
        """
        pass
    
    @abstractmethod
    def fetch_files(self, remote_dir_path: str, local_dir_path: str, pattern: Optional[str] = None) -> List[str]:
        """
        Fetch files from remote server
        
        Args:
            remote_dir_path: Directory on remote server
            local_dir_path: Directory to store downloaded files
            pattern: Optional file pattern to match
        
        Returns:
            List of successfully downloaded files
        """
        pass
