import os
import paramiko
import fnmatch
import stat
from typing import List, Optional, Tuple
import logging
from pathlib import Path

from .protocol_interface import FileTransferProtocol

class SSHProtocol(FileTransferProtocol):
    """
    Implementation of SSH/SFTP file transfer protocol
    """
    def __init__(self, host: str, username: str, password: str, port: int = 22):
        super().__init__(host, username, password, port)
        self.client = None
        self.sftp = None
        self.connected = False
        
    def connect(self) -> bool:
        """
        Establish SSH connection and open SFTP channel
        """
        try:
            self.logger.info(f"Connecting to {self.username}@{self.host}:{self.port}")
            self.client = paramiko.SSHClient()
            self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            self.client.connect(
                hostname=self.host,
                port=self.port,
                username=self.username,
                password=self.password,
                timeout=30
            )
            
            self.sftp = self.client.open_sftp()
            self.connected = True
            self.logger.info("SSH connection established successfully")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to establish SSH connection: {str(e)}")
            self.disconnect()
            return False
    
    def disconnect(self) -> None:
        """
        Close SFTP channel and SSH connection
        """
        try:
            if self.sftp:
                self.sftp.close()
                self.sftp = None
                
            if self.client:
                self.client.close()
                self.client = None
                
            self.connected = False
            self.logger.info("SSH connection closed")
            
        except Exception as e:
            self.logger.error(f"Error disconnecting SSH: {str(e)}")
    
    def _ensure_connection(self) -> bool:
        """
        Ensure connection is established
        """
        if not self.connected or not self.client or not self.sftp:
            return self.connect()
        return True
    
    def _ensure_remote_dir(self, remote_dir_path: str) -> bool:
        """
        Ensure remote directory exists, create if it doesn't
        """
        if not self._ensure_connection():
            return False
            
        try:
            dirs_to_create = []
            path_parts = remote_dir_path.rstrip('/').split('/')
            current_path = ""
            
            # Build the path incrementally and check each component
            for part in path_parts:
                if not part:
                    continue
                    
                if current_path:
                    current_path = f"{current_path}/{part}"
                else:
                    current_path = part
                
                try:
                    self.sftp.stat(current_path)
                except FileNotFoundError:
                    dirs_to_create.append(current_path)
            
            # Create directories that don't exist
            for dir_path in dirs_to_create:
                self.logger.info(f"Creating remote directory: {dir_path}")
                self.sftp.mkdir(dir_path)
                
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to ensure remote directory {remote_dir_path}: {str(e)}")
            return False
    
    def send_file(self, local_file_path: str, remote_dir_path: str) -> bool:
        """
        Send file to remote server
        
        Args:
            local_file_path: Path to local file
            remote_dir_path: Directory on remote server
        
        Returns:
            True if successful, False otherwise
        """
        if not self._ensure_connection():
            return False
            
        try:
            # Ensure local file exists
            if not os.path.isfile(local_file_path):
                self.logger.error(f"Local file not found: {local_file_path}")
                return False
            
            # Get filename and create branchwise directory
            filename = os.path.basename(local_file_path)
            file_tokens = filename.sp lit('-', 1)
            
            if len(file_tokens) < 2:
                self.logger.warning(f"Filename {filename} doesn't match expected format (branch-filename)")
                branch_dir = "default"
                remote_filename = filename
            else:
                branch_dir = file_tokens[0]
                remote_filename = file_tokens[1]
            
            # Ensure remote directory structure exists
            branch_path = f"{remote_dir_path.rstrip('/')}/{branch_dir}"
            if not self._ensure_remote_dir(branch_path):
                return False
            
            # Upload the file
            remote_file_path = f"{branch_path}/{remote_filename}"
            self.logger.info(f"Uploading {local_file_path} to {remote_file_path}")
            self.sftp.put(local_file_path, remote_file_path)
            
            self.logger.info(f"File {filename} uploaded successfully to {remote_file_path}")
            return True
            
        except Exception as e:
            self.logger.error(f"Failed to upload file {local_file_path}: {str(e)}")
            return False
    
    def fetch_files(self, remote_dir_path: str, local_dir_path: str, pattern: Optional[str] = None) -> List[str]:
        """
        Fetch files from remote server
        
        Args:
            remote_dir_path: Directory on remote server
            local_dir_path: Directory to store downloaded files
            pattern: Optional file pattern to match
        
        Returns:
            List of successfully downloaded files
        """
        if not self._ensure_connection():
            return []
            
        downloaded_files = []
        
        try:
            # Ensure local directory exists
            os.makedirs(local_dir_path, exist_ok=True)
            
            # Check if remote directory exists
            try:
                self.sftp.chdir(remote_dir_path)
            except FileNotFoundError:
                self.logger.warning(f"Remote directory not found: {remote_dir_path}")
                return []
            
            # Get current path to return to after operations
            current_path = self.sftp.getcwd()
            
            # Process subdirectories recursively
            self._process_remote_dir(current_path, local_dir_path, pattern, downloaded_files)
            
            return downloaded_files
            
        except Exception as e:
            self.logger.error(f"Failed to fetch files from {remote_dir_path}: {str(e)}")
            return downloaded_files
    
    def _process_remote_dir(self, remote_dir: str, local_dir: str, pattern: Optional[str], downloaded_files: List[str]) -> None:
        """
        Process remote directory recursively
        
        Args:
            remote_dir: Current remote directory path
            local_dir: Local directory to save files to
            pattern: File pattern to match
            downloaded_files: List to append downloaded file paths to
        """
        try:
            # List files in current directory
            items = self.sftp.listdir_attr(remote_dir)
            
            for item in items:
                remote_path = f"{remote_dir}/{item.filename}"
                
                # Skip . and .. directories
                if item.filename in ('.', '..'):
                    continue
                
                # Handle directories
                if stat.S_ISDIR(item.st_mode):
                    self.logger.debug(f"Found directory: {item.filename}")
                    
                    # Create corresponding local directory
                    local_subdir = os.path.join(local_dir, item.filename)
                    os.makedirs(local_subdir, exist_ok=True)
                    
                    # Process subdirectory recursively
                    self._process_remote_dir(remote_path, local_subdir, pattern, downloaded_files)
                
                # Handle files
                elif stat.S_ISREG(item.st_mode):
                    # Skip if pattern is specified and doesn't match
                    if pattern and not fnmatch.fnmatch(item.filename, pattern):
                        continue
                        
                    local_file_path = os.path.join(local_dir, item.filename)
                    
                    # Only download if file doesn't exist locally
                    if not os.path.exists(local_file_path):
                        self.logger.info(f"Downloading {remote_path} to {local_file_path}")
                        
                        try:
                            self.sftp.get(remote_path, local_file_path)
                            downloaded_files.append(local_file_path)
                        except Exception as e:
                            self.logger.error(f"Failed to download {remote_path}: {str(e)}")
                    else:
                        self.logger.debug(f"File already exists locally: {local_file_path}")
                        
        except Exception as e:
            self.logger.error(f"Error processing remote directory {remote_dir}: {str(e)}")
