import logging
from typing import List, Optional

from ..protocols.protocol_interface import FileTransferProtocol

class FileFetcher:
    """
    Service for fetching files from remote server
    """
    def __init__(self, protocol: FileTransferProtocol, remote_dir_path: str, 
                 local_dir_path: str, pattern: Optional[str] = None):
        self.protocol = protocol
        self.remote_dir_path = remote_dir_path
        self.local_dir_path = local_dir_path
        self.pattern = pattern
        self.logger = logging.getLogger("SSHFileToCbs.FileFetcher")
    
    def fetch_files(self) -> List[str]:
        """
        Fetch files from remote server
        
        Returns:
            List of downloaded files
        """
        self.logger.info(f"Starting to fetch files from {self.remote_dir_path} to {self.local_dir_path}")
        
        pattern_info = f" with pattern '{self.pattern}'" if self.pattern else ""
        self.logger.info(f"Fetching files from {self.remote_dir_path}{pattern_info}")
        
        # Connect to remote server
        if not self.protocol.connect():
            self.logger.error("Failed to connect to remote server")
            return []
        
        # Fetch files
        downloaded_files = []
        
        try:
            downloaded_files = self.protocol.fetch_files(
                self.remote_dir_path, 
                self.local_dir_path,
                self.pattern
            )
            
            self.logger.info(f"Downloaded {len(downloaded_files)} files")
            
        except Exception as e:
            self.logger.error(f"Error fetching files: {str(e)}")
            
        finally:
            # Always disconnect
            self.protocol.disconnect()
        
        return downloaded_files