import logging
from typing import List

from ..protocols.protocol_interface import FileTransferProtocol
from ..file_operations.file_manager import FileManager

class FileSender:
    """
    Service for sending local files to remote server
    """
    def __init__(self, protocol: FileTransferProtocol, local_folder_path: str, 
                 remote_file_path: str, archive_path: str):
        self.protocol = protocol
        self.local_folder_path = local_folder_path
        self.remote_file_path = remote_file_path
        self.archive_path = archive_path
        self.file_manager = FileManager()
        self.logger = logging.getLogger("SSHFileToCbs.FileSender")
    
    def send_files(self) -> int:
        """
        Send all files from local folder to remote server and archive them
        
        Returns:
            Number of files sent successfully
        """
        self.logger.info(f"Starting to send files from {self.local_folder_path} to {self.remote_file_path}")
        
        # Get list of files in local folder
        files = self.file_manager.get_files_in_directory(self.local_folder_path)
        
        if not files:
            self.logger.info(f"No files found in {self.local_folder_path}")
            return 0
        
        self.logger.info(f"Found {len(files)} files to send")
        
        # Connect to remote server
        if not self.protocol.connect():
            self.logger.error("Failed to connect to remote server")
            return 0
        
        # Send each file
        files_sent = 0
        
        try:
            for file_path in files:
                self.logger.info(f"Sending file: {file_path}")
                
                # Send file to remote server
                if self.protocol.send_file(file_path, self.remote_file_path):
                    # Archive file after successful transfer
                    if self.file_manager.move_file(file_path, self.archive_path):
                        files_sent += 1
                    else:
                        self.logger.error(f"Failed to archive file {file_path}")
                else:
                    self.logger.error(f"Failed to send file {file_path}")
        
        finally:
            # Always disconnect
            self.protocol.disconnect()
        
        self.logger.info(f"Completed sending files. {files_sent} of {len(files)} files sent successfully.")
        return files_sent