import configparser
import os
from typing import Dict, Any, Optional
import logging

class Config:
    """
    Configuration utility for loading and accessing application settings
    """
    def __init__(self, config_path: Optional[str] = None):
        self.logger = logging.getLogger("SSHFileToCbs.Config")
        self.config = configparser.ConfigParser()
        
        if config_path is None:
            # Default config path relative to this file
            self.config_path = os.path.join(
                os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
                'config',
                'config.ini'
            )
        else:
            self.config_path = config_path
            
        self.load_config()
    
    def load_config(self) -> None:
        """Load configuration from file"""
        try:
            if not os.path.exists(self.config_path):
                self.logger.error(f"Config file not found: {self.config_path}")
                raise FileNotFoundError(f"Config file not found: {self.config_path}")
                
            self.config.read(self.config_path)
            self.logger.info(f"Configuration loaded from {self.config_path}")
        except Exception as e:
            self.logger.error(f"Failed to load configuration: {str(e)}")
            raise
    
    def get(self, section: str, key: str, fallback: Any = None) -> Any:
        """Get config value by section and key"""
        try:
            return self.config.get(section, key)
        except (configparser.NoSectionError, configparser.NoOptionError) as e:
            self.logger.warning(f"Config value not found for {section}.{key}, using fallback: {fallback}")
            return fallback

    def get_int(self, section: str, key: str, fallback: Optional[int] = None) -> Optional[int]:
        """Get integer config value"""
        try:
            return self.config.getint(section, key)
        except (configparser.NoSectionError, configparser.NoOptionError, ValueError) as e:
            self.logger.warning(f"Failed to get integer config for {section}.{key}, using fallback: {fallback}")
            return fallback

    def get_all(self) -> Dict[str, Dict[str, str]]:
        """Get entire configuration as nested dict"""
        result = {}
        for section in self.config.sections():
            result[section] = dict(self.config[section])
        return result