import logging
import time
import psutil
import platform
import os
from typing import Dict, Any, List, Optional
import socket
import threading
from datetime import datetime

class Monitoring:
    """
    Monitoring class for tracking application health and performance
    """
    def __init__(self, app_name: str = "SSHFileToCbs"):
        self.app_name = app_name
        self.logger = logging.getLogger(f"{app_name}.Monitoring")
        self.start_time = time.time()
        self.metrics: Dict[str, Any] = {}
        self._lock = threading.Lock()
        self._background_thread: Optional[threading.Thread] = None
        self._stop_thread = threading.Event()
        
        # Initialize metrics
        self._collect_system_info()
    
    def _collect_system_info(self) -> None:
        """Collect system information for monitoring"""
        with self._lock:
            self.metrics["hostname"] = socket.gethostname()
            self.metrics["os"] = platform.system()
            self.metrics["os_version"] = platform.version()
            self.metrics["python_version"] = platform.python_version()
            self.metrics["cpu_count"] = psutil.cpu_count(logical=True)
            self.metrics["start_time"] = datetime.fromtimestamp(self.start_time).isoformat()
            
            try:
                self.metrics["ip_address"] = socket.gethostbyname(socket.gethostname())
            except:
                self.metrics["ip_address"] = "Unknown"
    
    def _collect_resource_usage(self) -> None:
        """Collect current resource usage"""
        with self._lock:
            process = psutil.Process(os.getpid())
            
            self.metrics["memory_usage_mb"] = process.memory_info().rss / (1024 * 1024)
            self.metrics["cpu_percent"] = process.cpu_percent(interval=0.1)
            self.metrics["threads_count"] = threading.active_count()
            self.metrics["uptime_seconds"] = time.time() - self.start_time
            self.metrics["last_updated"] = datetime.now().isoformat()
    
    def start_background_monitoring(self, interval: int = 60) -> None:
        """
        Start background monitoring in a separate thread
        
        Args:
            interval: Monitoring interval in seconds
        """
        if self._background_thread and self._background_thread.is_alive():
            self.logger.warning("Background monitoring is already running")
            return
            
        def _monitoring_thread() -> None:
            self.logger.info(f"Starting background monitoring with interval {interval} seconds")
            
            while not self._stop_thread.is_set():
                try:
                    self._collect_resource_usage()
                    self.log_metrics()
                except Exception as e:
                    self.logger.error(f"Error in monitoring thread: {str(e)}")
                
                # Sleep for the interval, but check stop flag periodically
                for _ in range(interval):
                    if self._stop_thread.is_set():
                        break
                    time.sleep(1)
        
        self._stop_thread.clear()
        self._background_thread = threading.Thread(target=_monitoring_thread, daemon=True)
        self._background_thread.start()
    
    def stop_background_monitoring(self) -> None:
        """Stop background monitoring thread"""
        if self._background_thread and self._background_thread.is_alive():
            self._stop_thread.set()
            self._background_thread.join(timeout=5)
            self.logger.info("Background monitoring stopped")
    
    def get_metrics(self) -> Dict[str, Any]:
        """
        Get current metrics
        
        Returns:
            Dictionary of metrics
        """
        with self._lock:
            # Update resource usage before returning
            self._collect_resource_usage()
            return self.metrics.copy()
    
    def log_metrics(self) -> None:
        """Log current metrics"""
        with self._lock:
            metrics = self.get_metrics()
            
            self.logger.info(
                f"System metrics - "
                f"Memory: {metrics.get('memory_usage_mb', 0):.2f} MB, "
                f"CPU: {metrics.get('cpu_percent', 0):.1f}%, "
                f"Threads: {metrics.get('threads_count', 0)}, "
                f"Uptime: {metrics.get('uptime_seconds', 0):.1f} seconds"
            )
    
    def record_operation(self, operation: str, status: str, details: Dict[str, Any]) -> None:
        """
        Record an operation for monitoring
        
        Args:
            operation: Name of the operation
            status: Status of the operation (success/failure)
            details: Details about the operation
        """
        timestamp = datetime.now().isoformat()
        
        with self._lock:
            if "operations" not in self.metrics:
                self.metrics["operations"] = []
                
            operation_record = {
                "timestamp": timestamp,
                "operation": operation,
                "status": status,
                **details
            }
            
            self.metrics["operations"].append(operation_record)
            
            # Keep only the latest 100 operations
            if len(self.metrics["operations"]) > 100:
                self.metrics["operations"] = self.metrics["operations"][-100:]
        
        log_level = logging.INFO if status == "success" else logging.ERROR
        self.logger.log(log_level, f"Operation '{operation}' {status}: {details}")